(in-package :snmp)

(defparameter *mib-list*
  '(#p"ASN-SNMP:AGENTX-MIB"
    #p"ASN-SNMP:BGP4-MIB"
    #p"ASN-SNMP:BRIDGE-MIB"
    #p"ASN-SNMP:DISMAN-EVENT-MIB"
    #p"ASN-SNMP:DISMAN-SCHEDULE-MIB"
    #p"ASN-SNMP:DISMAN-SCRIPT-MIB"
    #p"ASN-SNMP:EtherLike-MIB"
    #p"ASN-SNMP:GNOME-SMI"
    #p"ASN-SNMP:HCNUM-TC"
    #p"ASN-SNMP:HOST-RESOURCES-MIB"
    #p"ASN-SNMP:HOST-RESOURCES-TYPES"
    #p"ASN-SNMP:IANA-ADDRESS-FAMILY-NUMBERS-MIB"
    #p"ASN-SNMP:IANA-LANGUAGE-MIB"
    #p"ASN-SNMP:IANA-RTPROTO-MIB"
    #p"ASN-SNMP:IANAifType-MIB"
    #p"ASN-SNMP:IF-INVERTED-STACK-MIB"
    #p"ASN-SNMP:IF-MIB"
    #p"ASN-SNMP:INET-ADDRESS-MIB"
    #p"ASN-SNMP:IP-FORWARD-MIB"
    #p"ASN-SNMP:IP-MIB"
    #p"ASN-SNMP:IPV6-ICMP-MIB"
    #p"ASN-SNMP:IPV6-MIB"
    #p"ASN-SNMP:IPV6-TC"
    #p"ASN-SNMP:IPV6-TCP-MIB"
    #p"ASN-SNMP:IPV6-UDP-MIB"
    #p"ASN-SNMP:LM-SENSORS-MIB"
    #p"ASN-SNMP:NET-SNMP-AGENT-MIB"
    #p"ASN-SNMP:NET-SNMP-EXAMPLES-MIB"
    #p"ASN-SNMP:NET-SNMP-EXTEND-MIB"
    #p"ASN-SNMP:NET-SNMP-MIB"
    #p"ASN-SNMP:NET-SNMP-TC"
    #p"ASN-SNMP:NET-SNMP-VACM-MIB"
    #p"ASN-SNMP:NOTIFICATION-LOG-MIB"
    #p"ASN-SNMP:OSPF-MIB"
    #p"ASN-SNMP:OSPF-TRAP-MIB"
    #p"ASN-SNMP:RFC-1215"
    #p"ASN-SNMP:RFC1155-SMI"
    #p"ASN-SNMP:RFC1213-MIB"
    #p"ASN-SNMP:RIPv2-MIB"
    #p"ASN-SNMP:RMON-MIB"
    #+ignore #p"ASN-SNMP:SMUX-MIB"
    #p"ASN-SNMP:SNMP-COMMUNITY-MIB"
    #p"ASN-SNMP:SNMP-FRAMEWORK-MIB"
    #p"ASN-SNMP:SNMP-MPD-MIB"
    #p"ASN-SNMP:SNMP-NOTIFICATION-MIB"
    #p"ASN-SNMP:SNMP-PROXY-MIB"
    #p"ASN-SNMP:SNMP-TARGET-MIB"
    #p"ASN-SNMP:SNMP-USER-BASED-SM-MIB"
    #p"ASN-SNMP:SNMP-USM-AES-MIB"
    #p"ASN-SNMP:SNMP-USM-DH-OBJECTS-MIB"
    #p"ASN-SNMP:SNMP-VIEW-BASED-ACM-MIB"
    #p"ASN-SNMP:SNMPv2-CONF"
    #p"ASN-SNMP:SNMPv2-MIB"
    #p"ASN-SNMP:SNMPv2-SMI"
    #p"ASN-SNMP:SNMPv2-TC"
    #p"ASN-SNMP:SNMPv2-TM"
    #p"ASN-SNMP:SOURCE-ROUTING-MIB"
    #p"ASN-SNMP:TCP-MIB"
    #p"ASN-SNMP:TRANSPORT-ADDRESS-MIB"
    #p"ASN-SNMP:UCD-DEMO-MIB"
    #p"ASN-SNMP:UCD-DISKIO-MIB"
    #p"ASN-SNMP:UCD-DLMOD-MIB"
    #p"ASN-SNMP:UCD-IPFWACC-MIB"
    #p"ASN-SNMP:UCD-SNMP-MIB"
    #p"ASN-SNMP:UDP-MIB"
    #p"ASN-HP:RFC-1212"))

(defvar *mib-list-file* #p"SNMP:MIB.LISP-EXPR")

(defun lisp-file (path)
  (merge-pathnames (make-pathname :name (pathname-name path)
                                  :type "LISP"
                                  :directory '(:relative "MIB"))
                   #p"SNMP:"))

(defun expr-file (path)
  (merge-pathnames (make-pathname :name (pathname-name path)
                                  :type "LISP-EXPR"
                                  :directory '(:relative "MIB"))
                   #p"SNMP:"))

(defun update-mib ()
  (let ((mib.lisp-expr '()))
    (dolist (i *mib-list*)
      (format t "; Compiling ~A~%" i)
      (compile-asn.1 i :to (lisp-file i))
      (let ((depends (with-open-file (s (expr-file i) :direction :input)
                       (read s))))
        (push `(:file ,(string-downcase
                        (pathname-name (lisp-file i)))
                :depends-on ,depends)
              mib.lisp-expr)))
    (with-open-file (s *mib-list-file*
                       :direction :output
                       :if-exists :supersede)
      (format s ";;;; -*- Mode: Lisp -*-~%;;;; Auto-generated by SNMP:UPDATE-MIB.LISP~%")
      (pprint (setf mib.lisp-expr (nreverse mib.lisp-expr)) s)
      (format s "~%"))
    (load #p"SNMP:SNMP.ASD")
    (pprint mib.lisp-expr)))
